//----------------------------------------------------------------------------
//  Project Test_At3dx_Dll
//  MicroMouse Productions
//  Copyright  1999-2019. All Rights Reserved.
//  Author: Wayne Hogue
//
// this class was created for easily transferring code between
// Borland C++ Builder and Microsoft Visual C++
//
//----------------------------------------------------------------------------
#ifndef at3dDataH
#define at3dDataH

#if !defined(_MSC_VER)
#define _MSC_VER 1
#endif

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#define LEN_LAYER_NAME       80
#define LEN_LAYER_LONGNAME   2 * LEN_LAYER_NAME + 8
#define LEN_XYZ_DELIMITERS   40
#define LEN_FILE_FNAME      200

typedef struct tag_XYZ {
  double x;
  double y;
  double z;
  } XYZ;

typedef struct tag_FLOAT_XYZ {
  float x;
  float y;
  float z;
  } FLOAT_XYZ;

typedef struct tag_XYZ_UV {
  double x;
  double y;
  double z;
  float u;
  float v;
  } XYZ_UV;

typedef struct tag_FLOAT_XYZ_UV {
  float x;
  float y;
  float z;
  float u;
  float v;
  } FLOAT_XYZ_UV;

typedef struct tag_EDGE {
  Int32 p0;
  Int32 p1;
  } EDGE;

typedef struct tag_TRIANGLE {
  Int32 p0;
  Int32 p1;
  Int32 p2;
  } TRIANGLE;

typedef struct tag_UV {
  float u;
  float v;
  } UV;

typedef struct tag_UV_TRI {
  float u0;
  float v0;
  float u1;
  float v1;
  float u2;
  float v2;
  } UV_TRI;

typedef struct tag_VNORMAL {
  float x;
  float y;
  float z;
  } VNORMAL;

typedef struct tag_VNORMAL_TRI {
  float x0;
  float y0;
  float z0;
  float x1;
  float y1;
  float z1;
  float x2;
  float y2;
  float z2;
  } VNORMAL_TRI;

typedef struct tag_VCOLOR {
  float red;
  float green;
  float blue;
  float alpha;
  } VCOLOR;

typedef struct tag_VCOLOR_TRI {
  float red0;
  float green0;
  float blue0;
  float alpha0;
  float red1;
  float green1;
  float blue1;
  float alpha1;
  float red2;
  float green2;
  float blue2;
  float alpha2;
  } VCOLOR_TRI;

typedef struct tag_RGB {
  float red;
  float green;
  float blue;
  } RGB;

typedef struct tag_RGBA {
  float red;
  float green;
  float blue;
  float alpha;
  } RGBA;

#define OGL_LIGHT_COUNT   8

typedef struct tag_OGL_LIGHT {
  Int32 enabledFlag;
  RGBA ambient;
  RGBA diffuse;
  RGBA specular;
  FLOAT_XYZ position;
  } OGL_LIGHT;

typedef struct tag_DATA_3D {
  struct tag_DATA_3D* lastData;
  struct tag_DATA_3D* nextData;
  char name[LEN_LAYER_NAME];
  char parentName[LEN_LAYER_NAME];
  //
  XYZ pivotPoint;
  //
  Int32 PCount3D;  // for 3D faces
  Int32 TCount;
  XYZ* xyz3D;
  FLOAT_XYZ* xyzFloat3D;
  TRIANGLE* xyzIndex;
  UV_TRI* uv_tri;
  VNORMAL_TRI* vnorm_tri;
  VCOLOR_TRI* vcolor_tri;
  RGB* rgb;
  //
  Int32 uvCount;
  Int32 vnormCount;
  Int32 vcolorCount;
  Int32 xyz_uvCount;
  Int32 uvPresentFlag;       // use with XYZ_UV
  Int32 spare1;              // padding for align
  UV* uv;
  VNORMAL* vnorm;
  VCOLOR* vcolor;
  XYZ_UV* xyz_uv;
  FLOAT_XYZ_UV* xyzFloat_uv;
  TRIANGLE* uvIndex;
  TRIANGLE* vnormIndex;
  TRIANGLE* vcolorIndex;
  //
  Int32 PCount2D;  // for 2D edges
  Int32 ECount2D;
  XYZ* xyz2D;
  FLOAT_XYZ* xyzFloat2D;
  EDGE* edge2D;
  //
  Int32 spare2;       // padding for align
  Int32 PCountCloud;  // for point cloud (no edges or faces)
  XYZ* xyzCloud;
  FLOAT_XYZ* xyzFloatCloud;
  //
  Int32 dxfColor;
  Int32 doubleSided;
  Int32 rwxGeometrySampling;
  Int32 rwxLightSampling;
  Int32 rwxTextureModes;
  Int32 rwxClumpTag;
  Int32 rwxFaceTag;
  float ambient1;
  RGB ambientRGB;
  RGB colorRGB;
  float diffuse1;
  RGB diffuseRGB;
  float reflection1;
  RGB reflectionRGB;
  float specular1;
  RGB specularRGB;
  RGB emissiveRGB;
  float roughness;
  float shininess;
  float transparency1;
  RGB transparencyRGB;
  float indexRefraction;
  float creaseAngle;
  char texturePathName[MAX_PATH];
  char maskPathName[MAX_PATH];
  } DATA_3D;


typedef struct tag_OBJECT_MINMAX {
  double minX;
  double maxX;
  double minY;
  double maxY;
  double minZ;
  double maxZ;
  } OBJECT_MINMAX;


typedef struct tag_LAYER_MINMAX {
  Int32 cloudVertexCount;
  Int32 lineVertexCount;
  Int32 meshVertexCount;
  Int32 lineCount;
  Int32 triangleCount;
  Int32 spare;             // padding for align
  //
  double minX;
  double maxX;
  double minY;
  double maxY;
  double minZ;
  double maxZ;
  } LAYER_MINMAX;


typedef struct tag_MESH_TO_LANDSCAPE {
  Int32 stretchXFlag;
  Int32 stretchYFlag;
  Int32 convertFlag;
  Int32 addCloudFlag;
  Int32 dropFacesFlag;
  Int32 addBaseFlag;
  Int32 addUVFlag;
  Int32 writeXYZFlag;
  //
  double minX;
  double maxX;
  double sizeX;
  //
  double minY;
  double maxY;
  double sizeY;
  //
  double minZ;
  double maxZ;
  double sizeZ;
  //
  double xSpacing;
  double ySpacing;
  //
  Int32 xCount;
  Int32 yCount;
  Int32 vertCount;
  Int32 edgeCount;
  Int32 triangleCount;
  Int32 spare;          // padding for align
  //
  double newSizeX;
  double newSizeY;
  double lastXSpacing;
  double lastYSpacing;
  //
  double dropElevation;
  double baseThickness;
  } MESH_TO_LANDSCAPE;


// values for transferFromTypeFlag
//
#define TRANSFERFROM_TYPE_BASIC     0
#define TRANSFERFROM_TYPE_ARRAYS    1
#define TRANSFERFROM_TYPE_XYZ_UV    2

#define BEVELS_ARRAY_TOTAL   20

#define BITMAP_IN_PATTERN     "*.bmp;*.gif;*.iff;*.jpg;*.jpeg;*.png;*.tga;*.tif"

class CAt3dData
{
public:
  CAt3dData();
  virtual ~CAt3dData();

  void Clear_All_Objects(void);
  Int32 Get_Read_FileName(void);
  void ReadFile(void);
  void ReadAndRotateFile(void);
  void WriteFile(void);
  DATA_3D* Allocate_Data_3D(void);
  Int32 Transfer_Face_Verts_FromDll(Int32 fastFlag, DATA_3D* currentData);
  Int32 Transfer_Face_Verts_ToDll(Int32 fastFlag, DATA_3D* currentData);
  Int32 Transfer_Face_UVs_FromDll(Int32 fastFlag, DATA_3D* currentData);
  Int32 Transfer_Face_UVs_ToDll(Int32 fastFlag, DATA_3D* currentData);
  Int32 Transfer_Face_VNormals_FromDll(Int32 fastFlag, DATA_3D* currentData);
  Int32 Transfer_Face_VNormals_ToDll(Int32 fastFlag, DATA_3D* currentData);
  Int32 Transfer_Face_VColors_FromDll(Int32 fastFlag, DATA_3D* currentData);
  Int32 Transfer_Face_VColors_ToDll(Int32 fastFlag, DATA_3D* currentData);
  Int32 Transfer_Face_Colors_FromDll(Int32 fastFlag, DATA_3D* currentData);
  Int32 Transfer_Face_Colors_ToDll(Int32 fastFlag, DATA_3D* currentData);
  Int32 Transfer_2DLines_FromDll(Int32 fastFlag, DATA_3D* currentData);
  Int32 Transfer_2DLines_ToDll(Int32 fastFlag, DATA_3D* currentData);
  Int32 Transfer_PointCloud_FromDll(Int32 fastFlag, DATA_3D* currentData);
  Int32 Transfer_PointCloud_ToDll(Int32 fastFlag, DATA_3D* currentData);
  void Transfer_Material_FromDll(DATA_3D* currentData);
  void Transfer_Material_ToDll(DATA_3D* currentData);
  void TransferFromDll(Int32 fastFlag);
  void TransferToDll(Int32 fastFlag);
  void ClearTransferData(void);
  void MergeVertices(void);
  void DeleteLayer(void);
  void MakeSmooth1(void);
  void MakeSmooth2(void);
  void EraseSmooth1(void);
  void EraseSmooth2(void);
  void SelectFont(void);
  void GetFontDirectory(void);
  void ConvertText(Int32 unicodeFlag);
  void GetDefaultMaterials(void);
  void StartStopAlign(void);
  void SelectAlignLayer(void);
  void FlipAlignLayer(void);
  void SpecialTransferFromDll(Int32 fastFlag);
  void SpecialTransferToDll(Int32 fastFlag);
  void TransferTextureSquareToDll1(void);
  void TransferTextureSquareToDll2(void);
  void TransferTextureSquareToDll3(void);
  void TransferRedKnightToDll(void);
  void TransferGreenKnightToDll(void);
  void TransferBlueKnightToDll(void);
  void TransferGreenMonkToDll(void);
  void TransferBlueMonkToDll(void);
  void TransferMultiMatKnightToDll(void);
  void Create_UV_Coordinates(void);
  void Remove_UV_Coordinates(void);
  void DivideTriangles(Int32 typeFlag);
  void WriteDemContourMap(void);
  void EraseLines1(void);
  void EraseLines2(void);
  void EraseClouds1(void);
  void EraseClouds2(void);
  void WatertightCheck(void);
  void EditGetUpperBevels(void);
  void EditSetUpperBevels(void);
  void EditGetLowerBevels(void);
  void EditSetLowerBevels(void);
  void ApplyBevels1(void);
  void ApplyBevels2(void);
  void ApplyBevels3(void);
  void Text_MakeBevelSets(void);  
  void GetStageImage(void);
  void StartReducePolygons(void);
  void SelectReducePolygonsLayer(void);
  void GetReducePolygonsLayerData(void);
  void DoReducePolygons(void);
  void GetBackgroundImage(void);
  void Create_snapshotRgbaArray(Int32 width, Int32 height);
  void Free_snapshotRgbaArray(void);
  void Create_snapshotDib(void);
  void Free_snapshotDib(void);
  void Transfer_Snapshot_To_Dib(void);
  void Display_snapshotDib(HWND destHwnd);
  void GetOglSnapshot( Int32 width, Int32 height, HWND destHwnd);
  void Calculate_DMS(double location, Int32* degreesPtr, Int32* minutesPtr,
                     double* secondsPtr, Int32* signPtr);
  double Combine_DMS(Int32 degrees, Int32 minutes, double seconds, Int32 sign);
  void Get_KML_Data_From_DLL(void);
  void Transfer_2D_Outlines(Int32 flag);
  void Read_DoStuff_Write_File(void);
  //
  HWND hwnd;  // used in MS VC++
  //
  Int32 dllOpen;
  //
  Int32 dataCount;
  DATA_3D* parentData;
  //
  Int32 fileLoaded;
  char inName[MAX_PATH];
  char outName[MAX_PATH];
  //
  Int32 readType;
  Int32 writeType;
  //
  Int32 spare1;                      // padding for align
  char alignName[LEN_LAYER_NAME];
  void* alignNextLayer;
  //
  char fontPath[MAX_PATH];
  char dxfFontName[LF_FACESIZE];
  char vrmlSerifFontName[LF_FACESIZE];
  char vrmlSansFontName[LF_FACESIZE];
  char vrmlTypewriterFontName[LF_FACESIZE];
  //
  char textText[256];
  wchar_t textUnicodeText[256];
  char textFaceName[LF_FACESIZE];
  Int32 textLayersFlag;
  Int32 textStencilFlag;
  Int32 textPlusCoreFlag;
  Int32 textReverseOutlinesFlag;
  Int32 textOnCurveFlag;
  Int32 textOrigVertsFlag;
  Int32 textAllCharsFlag;
  Int32 textReduceFlag;
  Int32 textJoinFlag;
  Int32 textFaceFlag;
  Int32 textExtrudeFlag;
  Int32 textBoldFlag;
  Int32 textItalicFlag;
  Int32 textPoints;
  Int32 spare2;                      // padding for align
  double textHeight;
  double textDepth;
  double textBorder;
  //
  Int32 vrmlTextFlag;
  //
  Int32 outputAvatarFlag;
  Int32 outputHierarchyFlag;
  Int32 oneLayerPerFileFlag;
  Int32 multiMaterialFlag;
  Int32 quadFlag;
  Int32 uvFlag;
  Int32 vnormFlag;
  Int32 vcolorFlag;
  Int32 savePointCloudFlag;
  Int32 save2DFlag;
  Int32 autoDivideLayersFlag;
  Int32 noIndentFlag;
  Int32 textAsTextFlag;
  //
  Int32 bufferReadFlag;
  Int32 bufferWriteFlag;
  //
  Int32 transferFromTypeFlag;
  Int32 vertexFloatTransferFlag;
  //
  Int32 removeVerticesFlag;
  Int32 removeFacesFlag;
  //
  Int32 useDxfTextFlag;
  Int32 dxfAllLayersOnFlag;
  //
  Int32 outType3dmf;
  Int32 pixelFlag3dmf;
  //
  Int32 showReadMessagesFlag;
  Int32 showSaveMessagesFlag;
  //
  double triQuadNormalAngle;
  double triQuadEdgeAngle;
  //
  Int32 useOglFlag;
  Int32 oglDebugMsgFlag;
  Int32 pauseAtOglSnapshotMsgFlag;
  Int32 twoSidedLightingFlag;
  Int32 cullBackFacesFlag;
  Int32 displayTexturesFlag;
  Int32 defaultTextureFlag;
  Int32 oglReplaceFlag;
  Int32 useWhiteBaseFlag;
  Int32 textureSearchFlag;
  Int32 textureNameMatchFlag;
  Int32 relocateTexturesFlag;
  Int32 saveTexturePathFlag;
  Int32 partialTexturePathFlag;
  //
  double cameraXViewpoint;
  double cameraYViewpoint;
  double cameraZViewpoint;
  //
  double cameraXAngle;
  double cameraYAngle;
  double cameraZAngle;
  double cameraStepAngle;
  //
  double cameraInitialDistance;
  double cameraDistance;
  //
  Int32 cameraRotateSpeed;
  Int32 cameraZoomSpeed;
  Int32 reverseZoomDirectionFlag;
  Int32 reversePanDirectionFlag;
  //
  double cameraFieldOfView;
  double cameraStepFOV;
  //
  Int32 cameraXPan;
  Int32 cameraZPan;
  Int32 cameraStepPan;
  Int32 spare3;                      // padding for align
  //
  OBJECT_MINMAX objectMinMax;
  XYZ objectCenter;
  XYZ objectEdit;
  Int32 originRotateFlag;
  Int32 drawObjectAxesFlag;
  //
  float defaultAmbient1;
  RGB defaultAmbientRGB;
  RGB defaultColorRGB;
  float defaultDiffuse1;
  RGB defaultDiffuseRGB;
  float defaultSpecular1;
  RGB defaultSpecularRGB;
  RGB defaultEmissiveRGB;
  float defaultRoughness;
  float defaultShininess;
  float defaultTransparency1;
  RGB defaultTransparencyRGB;
  float defaultIndexOfRefraction;
  float defaultCreaseAngle;
  Int32 defaultDxfColor;
  Int32 defaultDoubleSided;
  Int32 defaultRwxGeometrySampling;
  Int32 defaultRwxLightSampling;
  Int32 defaultRwxTextureModes;
  Int32 defaultRwxClumpTag;
  Int32 defaultRwxFaceTag;
  //
  //Int32 spare4;                      // padding for align
  Int32 inCount;
  Int32 inIndex;
  Int32 inType[51];
  const char* inLabel[51];
  const char* inExt[51];
  //
  Int32 outCount;
  Int32 outIndex;
  Int32 outType[46];
  const char* outLabel[46];
  const char* outExt[46];
  //
  Int32 oglRenderCount;
  Int32 oglRenderIndex;
  Int32 oglRenderType[10];
  const char* oglRenderLabel[10];
  //
  Int32 spare5;                      // padding for align
  Int32 backgroundCount;
  Int32 backgroundIndex;
  Int32 backgroundType[5];
  const char* backgroundLabel[5];
  RGB backgroundColor;
  //
  Int32 spare6;                      // padding for align
  float textureCenter[2];
  float textureRotation;
  float textureScale[2];
  float textureTranslation[2];
  Int32 textureTypeIndex;
  Int32 textureTypeCount;
  Int32 textureType[3];
  const char* textureTypeLabel[3];
  //
  Int32 useFilmstripFlag;
  Int32 filmstripTimerFlag;
  //
  Int32 useAmbient1Flag;
  Int32 useDiffuse1Flag;
  Int32 useReflection1Flag;
  Int32 useSpecular1Flag;
  Int32 useTransparency1Flag;
  //
  Int32 demBitsPerPixel;
  Int32 demColumns;
  Int32 demRows;
  double demXSpacing;
  double demYSpacing;
  double demMinHeight;
  double demMaxHeight;
  Int32 demXOffset;
  Int32 demYOffset;
  Int32 demSeaLevelFlag;
  float demBelowFactor;
  float demAboveFactor;
  //
  Int32 spare7;                      // padding for align
  Int32 extrudeModeFlag;
  Int32 bevelSetsFlag;
  char textName[LEN_LAYER_NAME];
  void* textNextLayer;
  //
  char editName[LEN_LAYER_NAME];
  void* editNextLayer;
  //
  double editMoveVertexX;
  double editMoveVertexY;
  double editMoveVertexZ;
  double editMoveVertexStep;
  //
  double editRotateXAngle;
  double editRotateYAngle;
  double editRotateZAngle;
  double editRotateStepAngle;
  //
  double editScaleXFactor;
  double editScaleYFactor;
  double editScaleZFactor;
  double editScaleStepFactor;
  //
  double editGridSize;
  //
  Int32 editSnapKeepRelativeFlag;
  //
  Int32 spare8;                      // padding for align
  Int32 upperBevelsCount;
  Int32 lowerBevelsCount;
  double upperBevelsWidth[BEVELS_ARRAY_TOTAL];
  double upperBevelsHeight[BEVELS_ARRAY_TOTAL];
  double lowerBevelsWidth[BEVELS_ARRAY_TOTAL];
  double lowerBevelsHeight[BEVELS_ARRAY_TOTAL];
  //
  MESH_TO_LANDSCAPE meshToLandscape;
  //
  Int32 extrude2D_open;
  Int32 extrude2D_typeFlag;
  double extrude2D_minMax[6];
  double extrude2D_xRotation;
  double extrude2D_yRotation;
  double extrude2D_zRotation;
  double extrude2D_thickness;
  double extrude2D_adjustLocation;
  double extrude2D_stepAngle;
  void* extrude2D_layerPtr;
  char extrude2D_layerName[LEN_LAYER_NAME];
  //
  Int32 vrmlByChiefArchitectFlag;
  Int32 vrmlMiterJointExtrusionFlag;
  //
  Int32 kerkytheaUseRadianceFlag;
  Int32 kerkytheaRadianceEmitterFlag;
  Int32 kerkytheaUsePngAlphaMasksFlag;
  Int32 kerkytheaDirectionalAsSpotFlag;
  Int32 kerkytheaPointAsSpotFlag;
  Int32 kerkytheaPointUpFlag;
  Int32 kerkytheaDirectionalAttenuationFlag;
  Int32 kerkytheaPointAttenuationFlag;
  Int32 kerkytheaSpotAttenuationFlag;
  Int32 kerkytheaSpotReplaceFlag;
  Int32 kerkytheaSpotUseHotFractionFlag;

  Int32 kerkytheaDirectionalShadowFlag;
  Int32 kerkytheaDirectionalSoftShadowFlag;
  Int32 kerkytheaDirectionalGlobalPhotonsFlag;
  Int32 kerkytheaDirectionalCausticPhotonsFlag;
  Int32 kerkytheaDirectionalNegativeFlag;

  Int32 kerkytheaPointShadowFlag;
  Int32 kerkytheaPointSoftShadowFlag;
  Int32 kerkytheaPointGlobalPhotonsFlag;
  Int32 kerkytheaPointCausticPhotonsFlag;
  Int32 kerkytheaPointNegativeFlag;

  Int32 kerkytheaSpotShadowFlag;
  Int32 kerkytheaSpotSoftShadowFlag;
  Int32 kerkytheaSpotGlobalPhotonsFlag;
  Int32 kerkytheaSpotCausticPhotonsFlag;
  Int32 kerkytheaSpotNegativeFlag;

  float kerkytheaDirectionalMultiplier;
  float kerkytheaPointMultiplier;
  float kerkytheaSpotMultiplier;
  float kerkytheaDirectionalFallOff;
  float kerkytheaPointFallOff;
  float kerkytheaSpotFallOff;
  float kerkytheaDirectionalHotSpot;
  float kerkytheaPointHotSpot;
  float kerkytheaSpotHotSpot;
  float kerkytheaSpotHotFraction;
  float kerkytheaDirectionalRadius;
  float kerkytheaPointRadius;
  float kerkytheaSpotRadius;
  //
  char xyzDelimiters[LEN_XYZ_DELIMITERS];
  Int32 xyzSkipDlgFlag;     // if set, do not use XyzDialog
  Int32 xyzScanFlag;        // 0 = row and column mesh, 1 = probe scan,
                            // 2 = double Z scan
  Int32 xyzBinaryFileFlag;  // 0 = ASCII, 1 = binary
  Int32 xyzMotorolaFlag;    // 0 = Intel, 1 = Motorola
  Int32 xyzDoubleFlag;      // 0 = float, 1 = double
  Int32 xyzRows;            // number of rows
  Int32 xyzColumns;         // number of columns
  Int32 xyzByColumnFlag;    // 0 = by row, 1 = by column
  Int32 xyzClipFlag;
  Int32 xyzFromRow;
  Int32 xyzToRow;
  Int32 xyzFromColumn;
  Int32 xyzToColumn;
  Int32 xyzAxisFlag;        // axis system
  Int32 xyzSaveDxfFlag;
  Int32 xyzDxfReleaseFlag;  // 0 = R10 Flat 0, 1 = R10 Flag 1, 2 = R11+
  Int32 xyzNoDataFlag;
  double xyzNoDataValue;
  double xyzNoDataReplaceValue;
  Int32 xyzNoDataReplaceFlag;
  Int32 xyzScaleZFlag;
  double xyzScaleFactorZ;
  Int32 xyzHasOptionsFlag;
  Int32 xyzUseIdFlag;
  Int32 xyzRgbIntegersFlag;
  Int32 xyzSkipCount;
  //
  Int32 spare9;                      // padding for align
  char stageImageName[MAX_PATH];
  Int32 stageUseFlag;
  Int32 stageOriginFlag;
  Int32 stageSquaresFlag;
  Int32 stageImageFlag;
  double stageSpacing;
  double stageWidth;
  double stageDepth;
  double stageOriginX;
  double stageOriginY;
  //
  char reducePolygonsName[LEN_LAYER_NAME];
  void* reducePolygonsNextLayer;
  Int32 reducePolygonsVertices0;
  Int32 reducePolygonsVertices1;
  Int32 reducePolygonsTriangles0;
  Int32 reducePolygonsTriangles1;
  Int32 reducePolygonsStep;
  //
  char backgroundImageName[MAX_PATH];
  Int32 backgroundImageUseFlag;
  //
  Int32 bitmapTo3D_SkipDlg_Flag;        // if set do not use BitmapTo3dDlg
  Int32 bitmapTo3D_Decode0_Flag;
  Int32 bitmapTo3D_SaveOutlines_Flag;
  Int32 bitmapTo3D_JoinOutlines_Flag;
  Int32 bitmapTo3D_AddFaces_Flag;
  double bitmapTo3D_ExtrudeHeight;
  Int32 bitmapTo3D_Extrude_Flag;
  //
  Int32 spare10;                      // padding for align
  Int32 bitmapToDem_SkipDlg_Flag;     // if set do not use DemMultDlg
  Int32 bitmapToDem_SeaLevel_Flag;
  double bitmapToDem_BelowMultiplier;
  double bitmapToDem_AboveMultiplier;
  double bitmapToDem_MinHeight;
  double bitmapToDem_XY_Spacing;
  //
  UInt8* snapshotRgbaArray;
  Int32 snapshotWidth;
  Int32 snapshotHeight;
  BITMAPINFO* snapshotBmInfoPtr;
  UInt8* snapshotImagePtr;
  HBITMAP snapshotHbitmap;
  //
  Int32 modoCameraSunlightFlag;
  Int32 modoHierarchyFlag;
  Int32 modoWithJointsFlag;
  Int32 spare11;                      // padding for align
  //
  char layerMinMaxName[LEN_LAYER_NAME];
  void* layerMinMaxNextLayer;
  //
  RGBA oglGlobalAmbient;
  OGL_LIGHT oglDirectionalLights[OGL_LIGHT_COUNT];
  Int32 oglSelectedLight;
  //
  // KML values for doc.kml for Google Earth
  //
  Int32 kmlModel_Altitude_Mode_Flag;
  Int32 kmlLookAt_Altitude_Mode_Flag;
  //
  Int32 kmlModel_Latitude_Degrees;
  Int32 kmlModel_Latitude_Minutes;
  Int32 kmlModel_Latitude_Sign;
  Int32 kmlModel_Longitude_Degrees;
  Int32 kmlModel_Longitude_Minutes;
  Int32 kmlModel_Longitude_Sign;
  Int32 spare12;                      // padding for align
  double  kmlModel_Latitude_Seconds;
  double  kmlModel_Longitude_Seconds;
  double  kmlModel_Altitude;
  double  kmlModel_Heading;
  double  kmlModel_Tilt;
  double  kmlModel_Roll;
  double  kmlModel_Scale_X;
  double  kmlModel_Scale_Y;
  double  kmlModel_Scale_Z;
  //
  Int32 kmlLookAt_Latitude_Degrees;
  Int32 kmlLookAt_Latitude_Minutes;
  double  kmlLookAt_Latitude_Seconds;
  Int32 kmlLookAt_Latitude_Sign;
  Int32 kmlLookAt_Longitude_Degrees;
  Int32 kmlLookAt_Longitude_Minutes;
  Int32 kmlLookAt_Longitude_Sign;
  double  kmlLookAt_Longitude_Seconds;
  double  kmlLookAt_Altitude;
  double  kmlLookAt_Heading;
  double  kmlLookAt_Tilt;
  double  kmlLookAt_Range;
  //
#ifdef USE_CPP_BUILDER
  AnsiString kmlFolderName;
  AnsiString kmlFolderDescription;
  AnsiString kmlPlacemarkName;
  AnsiString kmlPlacemarkDescription;
#else
  CString kmlFolderName;
  CString kmlFolderDescription;
  CString kmlPlacemarkName;
  CString kmlPlacemarkDescription;
#endif
  char kmlModelsFolderName[LEN_FILE_FNAME];
  char kmlImagesFolderName[LEN_FILE_FNAME];
  Int32 kmlUseModelsFolderFlag;
  Int32 kmlUseImagesFolderFlag;
  //
  char kmlAltitudeModeLabels[KML_ALTITUDE_MODE_COUNT][20];
  //
  Int32 colladaOutputUpAxisFlag;
  //
  Int32 grabKeyboardFocusFlag;
  Int32 alwaysShowCameraCursorFlag;
  Int32 touchScreenInputFlag;
  Int32 debugStopMessagesFlag;
  //
  Int32 checkFixTransparenciesFlag;
  Int32 kmzInputModelFactorsFlag;
  Int32 alwaysWriteColladaTransparencyFlag;
  Int32 spare13;                      // padding for align
  //
#ifdef USE_CPP_BUILDER
  AnsiString mayaVersion;
  AnsiString directxFramePrefix;
  AnsiString directxMaterialPrefix;
#else       
  CString mayaVersion;
  CString directxFramePrefix;
  CString directxMaterialPrefix;
#endif
  Int32 useDirectxFramePrefixFlag;
  Int32 useDirectxMaterialPrefixFlag;
  //
  Int32 xamlOutputClassFlag;
  //
  Int32 saveCloudColorDivideFlag;

}; // end class CAt3dData


#ifdef USE_CPP_BUILDER
class CMyImagePanel : public TPanel
{
  private:
    CAt3dData* at3dData;
  protected:
    virtual void __fastcall CreateParams(TCreateParams &Params);
    virtual void __fastcall Paint();
  public:
    __fastcall CMyImagePanel(CAt3dData* dataPtr, TComponent* Owner);
};
#endif

#endif // at3dDataH

